@@ -31,6 +31,15 @@ def prepare_for_peppercorn(handler_instance, defaults, errors, state,
     return fill_kwargs
 
 
+def set_css_classes(override_css_classes=None):
+    def _set_css_classes(handler_instance, defaults, errors, state, fill_kwargs):
+        # Only override them if we were given some classes.
+        if override_css_classes:
+            fill_kwargs['css_classes'] = override_css_classes
+        return fill_kwargs
+    return _set_css_classes
+
+        
 def set_closed(closed):
     """
     Create a closure to set a boolean in the fillkwargs to close tags or not.
