@@ -27,7 +27,7 @@ About render_kwargs
 Field Extras
 ============
 
-  * What is an example?
+  * What is an example of an example?
 
     Date Ex: 10-11-1982
 
@@ -45,6 +45,7 @@ error and a special clear part.  The clear part just inject a div with
 a clear class in order to allow floats to be cleared.  This part can occur
 more than once in the given order.
 """
+from types import StringTypes
 from mako.runtime import supports_caller, capture
 from webhelpers.misc import NotGiven
 from webhelpers.html.builder import HTML
@@ -101,6 +102,7 @@ default_css_classes = {
     'clear': 'clear',
     'primary_form_action': 'primary_form_action',
     'secondary_form_action': 'secondary_form_action',
+    'field_has_error': 'field_has_error',
 }
 """
 Notable css classes indexed by key.
@@ -109,16 +111,44 @@ Overwrite these in the render kwargs.
 """
 
 
-def _determine_css_class(render_kwargs, class_name):
-    css_class = None
+def _to_css_class(css_classes):
+    return ' '.join(css_classes)
+
+
+def _determine_css_classes(render_kwargs, class_name,
+            add_css_classes=None):
+    """
+    Determine the css classes to use by first checking the dict at the
+        `css_classes` key in render_kwargs and then checking defaults in this
+        module.
+    """
+    css_classes = None
     if 'css_classes' in render_kwargs:
-        css_class = render_kwargs['css_classes'].get(class_name, None)
-    if not css_class:
-        css_class = default_css_classes.get(class_name, None)
-    return css_class
+        css_classes = render_kwargs['css_classes'].get(class_name, None)
+    if not css_classes:
+        css_classes = default_css_classes.get(class_name, None)
+    # Wart to handle css classes as a string.
+    if isinstance(css_classes, StringTypes):
+        css_classes = set([css_classes])
+    # Wart to handle css classes as a list or tuple.
+    elif not isinstance(css_classes, set):
+        css_classes = set(css_classes)
+    if add_css_classes:
+        css_classes.update(add_css_classes)
+    return css_classes
 
 
 def _determine_field_part_list(render_kwargs, field_parts, field_part_order, field_type=None):
+    """
+    Determine the order of the parts, such as label and input, in the HTML.
+
+    Note: This can be overridden in the render_kwargs by the field_type in a
+        dict at the `field_part_orders` key, use the field_type `None` to match
+        all unmatched field types. If no field part order is found in the
+        dict in render kwargs then defaults in this module are used.
+
+    A special field part exists to to inject a clearing div.
+    """
     if not field_part_order:
         if 'field_part_orders' in render_kwargs:
             if field_type:
@@ -158,16 +188,20 @@ def _render_field_errors(render_kwargs, name):
                 error_spans.append(tag('span', error))
         else:
             error_spans.append(tag('span', errors))
-        css_class = _determine_css_class(render_kwargs, 'field_errors')
+        css_class = _to_css_class(_determine_css_classes(render_kwargs,
+                'field_errors'))
         error_div = tag('div', *error_spans, class_=css_class)
     return error_div
 
 
 def _determine_extra_field_parts(render_kwargs, field_parts, field_extras):
-    help_css_class = _determine_css_class(render_kwargs, 'field_help')
-    example_css_class = _determine_css_class(render_kwargs, 'field_example')
+    """
+    Build any extra field parts as necessary, such as an example or help.
+    """
     extra_field_parts = {}
     if field_extras and 'example' in field_extras:
+        example_css_class = _to_css_class(_determine_css_classes(render_kwargs,
+                'field_example'))
         if 'extra_renderers' in render_kwargs and 'example' in \
                 render_kwargs['extra_renderers']:
             renderer = render_kwargs['extra_renderers']['example']
@@ -175,9 +209,14 @@ def _determine_extra_field_parts(render_kwargs, field_parts, field_extras):
                     class_=example_css_class,
                     example=field_extras['example'])
         else:
-            extra_field_parts['example'] = tag('span', field_extras['example'],
+            example = field_extras['example']
+            if type(example) not in (list, tuple):
+                example = [example]
+            extra_field_parts['example'] = tag('span', *example,
                     class_=example_css_class)
     if field_extras and 'help' in field_extras:
+        help_css_class = _to_css_class(_determine_css_classes(render_kwargs,
+                'field_help'))
         if 'extra_renderers' in render_kwargs and 'help' in \
                 render_kwargs['extra_renderers']:
             renderer = render_kwargs['extra_renderers']['help']
@@ -189,14 +228,12 @@ def _determine_extra_field_parts(render_kwargs, field_parts, field_extras):
     return extra_field_parts
 
 
-def primary_form_action(context, render_kwargs, value, css_classes=None, **attrs):
+def primary_form_action(context, render_kwargs, value, css_classes=None,
+            **attrs):
     """ Primary submit button -- Use this for the main action of the form. """
-    if not css_classes:
-        css_classes = []
-    else:
-        css_classes = css_classes[:]
-    css_class = _determine_css_class(render_kwargs, 'primary_form_action')
-    attrs['class_'] = ' '.join(css_classes + [css_class])
+    attrs['class_'] = _to_css_class(_determine_css_classes(render_kwargs,
+            'primary_form_action',
+            add_css_classes=css_classes))
     attrs.setdefault('type', 'submit')
     closed = render_kwargs.get('closed', False)
     context.write(tag('input', value=value, _closed=closed, **attrs))
@@ -208,8 +245,8 @@ def secondary_form_action(context, render_kwargs, value, tag_name='input', **att
     Seconary submit button -- Use this for an action that is less significant
     than the primary action.
     """
-    attrs['class_'] = _determine_css_class(render_kwargs,
-            'secondary_form_action')
+    attrs['class_'] = _to_css_class(_determine_css_classes(render_kwargs,
+            'secondary_form_action'))
     if tag_name == 'input':
         attrs.setdefault('type', 'button')
         attrs.setdefault('value', value)
@@ -220,16 +257,17 @@ def secondary_form_action(context, render_kwargs, value, tag_name='input', **att
     context.write(tag(tag_name, **attrs))
     return ''
 
-    
+
 def hidden_field(context, render_kwargs, name, value=NotGiven, id=NotGiven,
         field_part_order=None, **attrs):
+    """ Standard input element of type hidden. """
     if value is NotGiven:
         value = field_value(render_kwargs, name)
     if id is NotGiven:
         id = build_id(render_kwargs, name)
     attrs['_closed'] = render_kwargs.get('closed', False)
-    
-    attrs['class_'] = _determine_css_class(render_kwargs, 'hidden_field')
+    attrs['class_'] = _to_css_class(_determine_css_classes(render_kwargs,
+            'hidden_field'))
     field_parts = {}
     field_parts['input'] = tag('input', name=name, value=value, id=id, type='hidden', **attrs)
     error = _render_field_errors(render_kwargs, name)
@@ -243,6 +281,7 @@ def hidden_field(context, render_kwargs, name, value=NotGiven, id=NotGiven,
 
 def file_field(context, render_kwargs, name, value=NotGiven, id=NotGiven,
         label=NotGiven, field_extras=None, field_part_order=None, **attrs):
+    """ Standard input element of type file. """
     field_parts = {}
     if value is NotGiven:
         value = field_value(render_kwargs, name)
@@ -258,16 +297,21 @@ def file_field(context, render_kwargs, name, value=NotGiven, id=NotGiven,
     attrs['_closed'] = render_kwargs.get('closed', False)
     field_parts['input'] = tag('input', name=name, value=value, id=id, type='file', **attrs)
     error = _render_field_errors(render_kwargs, name)
+    add_css_classes = set()
     if error:
+        add_css_classes.update(_determine_css_classes(render_kwargs,
+            'field_has_error'))
         field_parts['error'] = error
     field_part_list = _determine_field_part_list(render_kwargs, field_parts, field_part_order)
-    css_class = _determine_css_class(render_kwargs, 'file_field')
+    css_class = _to_css_class(_determine_css_classes(render_kwargs,
+            'file_field', add_css_classes=add_css_classes))
     context.write(tag('div', *field_part_list, class_=css_class))
     return ''
 
 
 def password_field(context, render_kwargs, name, value=NotGiven, id=NotGiven,
         label=NotGiven, field_extras=None, field_part_order=None, **attrs):
+    """ Standard input element of type password. """
     if value is NotGiven:
         value = field_value(render_kwargs, name)
     if id is NotGiven:
@@ -282,17 +326,21 @@ def password_field(context, render_kwargs, name, value=NotGiven, id=NotGiven,
         field_parts['label'] = tag('label', label, for_=id)
     field_parts['input'] = tag('input', name=name, value=value, id=id, type='password', **attrs)
     error = _render_field_errors(render_kwargs, name)
+    add_css_classes = set()
     if error:
+        add_css_classes.update(_determine_css_classes(render_kwargs,
+            'field_has_error'))
         field_parts['error'] = error
     field_part_list = _determine_field_part_list(render_kwargs, field_parts, field_part_order)
-    css_class = _determine_css_class(render_kwargs, 'password_field')
+    css_class = _to_css_class(_determine_css_classes(render_kwargs,
+            'password_field', add_css_classes=add_css_classes))
     context.write(tag('div', *field_part_list, class_=css_class))
-    context.write('</div>')
     return ''
 
 
 def text_field(context, render_kwargs, name, value=NotGiven, id=NotGiven,
         label=NotGiven, field_extras=None, field_part_order=None, **attrs):
+    """ Standard input element of type text. """
     if value is NotGiven:
         value = field_value(render_kwargs, name)
     if id is NotGiven:
@@ -300,14 +348,18 @@ def text_field(context, render_kwargs, name, value=NotGiven, id=NotGiven,
     if label is NotGiven:
         label = name.replace(u'_', u' ').title()
     attrs['_closed'] = render_kwargs.get('closed', False)
-    css_class = _determine_css_class(render_kwargs, 'text_field')
     field_parts = {}
     if label:
         field_parts['label'] = tag('label', label, for_=id)
     field_parts['input'] = tag('input', name=name, value=value, id=id, type='text', **attrs)
     error = _render_field_errors(render_kwargs, name)
+    add_css_classes = set()
     if error:
+        add_css_classes.update(_determine_css_classes(render_kwargs,
+            'field_has_error'))
         field_parts['error'] = error
+    css_class = _to_css_class(_determine_css_classes(render_kwargs,
+            'text_field', add_css_classes=add_css_classes))
     if field_extras:
         field_parts.update(_determine_extra_field_parts(render_kwargs, field_parts, field_extras))
     field_part_list = _determine_field_part_list(render_kwargs, field_parts, field_part_order)
@@ -317,6 +369,7 @@ def text_field(context, render_kwargs, name, value=NotGiven, id=NotGiven,
 
 def checkbox_field(context, render_kwargs, name, checked=NotGiven, id=NotGiven,
         label=NotGiven, field_extras=None, field_part_order=None, **attrs):
+    """ Standard input element of type checkbox. """
     field_type = 'checkbox_field'
     if checked is NotGiven:
         checked = field_value(render_kwargs, name, False)
@@ -329,19 +382,23 @@ def checkbox_field(context, render_kwargs, name, checked=NotGiven, id=NotGiven,
     if label is NotGiven:
         label = name.replace(u'_', u' ').title()
     attrs['_closed'] = render_kwargs.get('closed', False)
-    css_class = _determine_css_class(render_kwargs, field_type)
     field_parts = {}
     field_parts['input'] = tag('input', name=name, value='1', checked=checked,
             id=id, type='checkbox', **attrs)
     if label:
         field_parts['label'] = tag('label', label, for_=id)
     error = _render_field_errors(render_kwargs, name)
+    add_css_classes = set()
     if error:
+        add_css_classes.update(_determine_css_classes(render_kwargs,
+            'field_has_error'))
         field_parts['error'] = error
     if field_extras:
         field_parts.update(_determine_extra_field_parts(render_kwargs, field_parts, field_extras))
     field_part_list = _determine_field_part_list(render_kwargs, field_parts,
             field_part_order, field_type=field_type)
+    css_class = _to_css_class(_determine_css_classes(render_kwargs, field_type,
+            add_css_classes=add_css_classes))
     context.write(tag('div', *field_part_list, class_=css_class))
     return ''
 
@@ -349,6 +406,11 @@ def checkbox_field(context, render_kwargs, name, checked=NotGiven, id=NotGiven,
 def textarea_field(context, render_kwargs, name, value=NotGiven, id=NotGiven,
         label=NotGiven, field_extras=None, field_part_order=None,
         field_classes=NotGiven, **attrs):
+    """ Standard textarea element. """
+    if field_classes is NotGiven:
+        add_css_classes = set()
+    else:
+        add_css_classes = set(field_classes)
     field_type = 'textarea_field'
     if value is NotGiven:
         value = field_value(render_kwargs, name, u'')
@@ -356,30 +418,27 @@ def textarea_field(context, render_kwargs, name, value=NotGiven, id=NotGiven,
         id = build_id(render_kwargs, name)
     if label is NotGiven:
         label = name.replace(u'_', u' ').title()
-    field_class = None
-    css_class = _determine_css_class(render_kwargs, field_type)
-    if field_classes is NotGiven and css_class:
-        field_class = css_class
-    elif field_classes:
-        if css_class:
-            field_classes = field_classes + [css_class]
-        field_class = ' '.join(field_classes)
     field_parts = {}
     if label:
         field_parts['label'] = tag('label', label, for_=id)
     field_parts['input'] = tag('textarea', value, name=name, id=id, **attrs)
     error = _render_field_errors(render_kwargs, name)
     if error:
+        add_css_classes.update(_determine_css_classes(render_kwargs,
+            'field_has_error'))
         field_parts['error'] = error
     if field_extras:
         field_parts.update(_determine_extra_field_parts(render_kwargs, field_parts, field_extras))
     field_part_list = _determine_field_part_list(render_kwargs, field_parts, field_part_order)
-    context.write(tag('div', *field_part_list, class_=field_class))
+    css_class = _to_css_class(_determine_css_classes(render_kwargs, field_type,
+            add_css_classes=add_css_classes))
+    context.write(tag('div', *field_part_list, class_=css_class))
     return ''
 
 
 def single_select_field(context, render_kwargs, name, options, value=NotGiven,
         id=NotGiven, label=NotGiven, field_extras=None, field_part_order=None, **attrs):
+    """ Standard select element where multiple is not set. """
     if value is NotGiven:
         value = field_value(render_kwargs, name, NotGiven)
     if value is NotGiven:
@@ -396,12 +455,16 @@ def single_select_field(context, render_kwargs, name, options, value=NotGiven,
     if label:
         field_parts['label'] = tag('label', label, for_=id)
     error = _render_field_errors(render_kwargs, name)
+    add_css_classes = set()
     if error:
+        add_css_classes.update(_determine_css_classes(render_kwargs,
+            'field_has_error'))
         field_parts['error'] = error
     if field_extras:
         field_parts.update(_determine_extra_field_parts(render_kwargs, field_parts, field_extras))
     field_part_list = _determine_field_part_list(render_kwargs, field_parts, field_part_order)
-    css_class = _determine_css_class(render_kwargs, 'single_select_field')
+    css_class = _to_css_class(_determine_css_classes(render_kwargs,
+            'single_select_field', add_css_classes=add_css_classes))
     context.write(tag('div', *field_part_list, class_=css_class))
     return ''
 
@@ -409,6 +472,7 @@ def single_select_field(context, render_kwargs, name, options, value=NotGiven,
 def multiple_select_field(context, render_kwargs, name, options,
         values=NotGiven, id=NotGiven, label=NotGiven, field_extras=None,
         field_part_order=None, **attrs):
+    """ Standard select element where multiple is set to true. """
     if values is NotGiven:
         values = field_value(render_kwargs, name, [])
     if id is NotGiven:
@@ -420,22 +484,25 @@ def multiple_select_field(context, render_kwargs, name, options,
     if label:
         field_parts['label'] = tag('label', label, for_=id)
     error = _render_field_errors(render_kwargs, name)
+    add_css_classes = set()
     if error:
+        add_css_classes.update(_determine_css_classes(render_kwargs,
+            'field_has_error'))
         field_parts['error'] = error
     field_parts['input'] = select(name, values, options, id=id, multiple=True)
     if field_extras:
         field_parts.update(_determine_extra_field_parts(render_kwargs, field_parts, field_extras))
     field_part_list = _determine_field_part_list(render_kwargs, field_parts, field_part_order)
-    css_class = _determine_css_class(render_kwargs, 'multiple_select_field')    
+    css_class = _to_css_class(_determine_css_classes(render_kwargs,
+            'multiple_select_field', add_css_classes=add_css_classes))
     context.write(tag('div', *field_part_list, class_=css_class))
     return ''
-    
 
 
 def multiple_select_list_field(context, render_kwargs, name, options,
         values=NotGiven, label=NotGiven, id=NotGiven, field_extras=None,
         field_part_order=None, **attrs):
-    """ Wrap a list of checkbox inputs. """
+    """ Wrap a list of input elements of type checkbox. """
     if values is NotGiven:
         values = field_value(render_kwargs, name, [])
     if id is NotGiven:
@@ -452,7 +519,8 @@ def multiple_select_list_field(context, render_kwargs, name, options,
     field_part_list.append(tag('input', type='hidden', name='__start__', value=start_token, _closed=closed))
     if label:
         field_parts['label'] = tag('legend', label)
-    item_css_class = _determine_css_class(render_kwargs, 'multiple_select_list_item')
+    item_css_class = _to_css_class(_determine_css_classes(render_kwargs,
+            'multiple_select_list_item'))
     field_parts['input'] = []
     for index, option in enumerate(options):
         (option_value, option_label) = option
@@ -466,14 +534,18 @@ def multiple_select_list_field(context, render_kwargs, name, options,
                         checked=checked, type='checkbox', **attrs),
                 tag('label', option_label, for_=option_id), class_=item_css_class))
     error = _render_field_errors(render_kwargs, name)
+    add_css_classes = set()
     if error:
+        add_css_classes.update(_determine_css_classes(render_kwargs,
+            'field_has_error'))
         field_parts['error'] = error
     if field_extras:
         field_parts.update(_determine_extra_field_parts(render_kwargs, field_parts, field_extras))
     field_part_list.extend(_determine_field_part_list(render_kwargs, field_parts, field_part_order))
     field_part_list.append(tag('input', type='hidden', name='__end__',
             value=start_token, _closed=closed))
-    css_class = _determine_css_class(render_kwargs, 'multiple_select_list_field')
+    css_class = _to_css_classes(_determine_css_classes(render_kwargs,
+            'multiple_select_list_field', add_css_classes=add_css_classes))
     context.write(tag('div', tag('fieldset', *field_part_list), class_=css_class))
     return ''
 
@@ -481,7 +553,7 @@ def multiple_select_list_field(context, render_kwargs, name, options,
 def single_select_list_field(context, render_kwargs, name, options,
         value=NotGiven, id=NotGiven, label=NotGiven, field_extras=None,
         field_part_order=None, **attrs):
-    """ Wrap a list of radio inputs. """
+    """ Wrap a list of input elements of type radio. """
     if value is NotGiven:
         value = field_value(render_kwargs, name)
     if id is NotGiven:
@@ -492,7 +564,8 @@ def single_select_list_field(context, render_kwargs, name, options,
     field_parts = {}
     if label:
         field_parts['label'] = tag('legend', label)
-    item_css_class = _determine_css_class(render_kwargs, 'single_select_list_item')
+    item_css_class = _to_css_class(_determine_css_classes(render_kwargs,
+            'single_select_list_item'))
     field_parts['input'] = []
     for index, option in enumerate(options):
         (option_value, option_label) = option
@@ -501,25 +574,32 @@ def single_select_list_field(context, render_kwargs, name, options,
             checked = 'checked'
         else:
             checked = None
-        option_id = id + u'_' + option_value        
+        option_id = id + u'_' + option_value
         field_parts['input'].append(tag('div',
                 tag('input', name=name, value=option_value, id=option_id,
                         checked=checked, type='radio', **attrs),
                 tag('label', option_label, for_=option_id),
                         class_=item_css_class))
     error = _render_field_errors(render_kwargs, name)
+    add_css_classes = set()
     if error:
+        add_css_classes.update(_determine_css_classes(render_kwargs,
+                'field_has_error'))
         field_parts['error'] = error
     if field_extras:
         field_parts.update(_determine_extra_field_parts(render_kwargs, field_parts, field_extras))
     field_part_list = _determine_field_part_list(render_kwargs, field_parts, field_part_order)
-    css_class = _determine_css_class(render_kwargs, 'single_select_list_field')
+    css_class = _to_css_class(_determine_css_classes(render_kwargs,
+            'single_select_list_field', add_css_classes=add_css_classes))
     context.write(tag('div', tag('fieldset', *field_part_list), class_=css_class))
     return ''
 
 
 @supports_caller
 def rename(context, render_kwargs, name=None):
+    """
+    Read the peppercorn documentation to understand why you need this.
+    """
     value = make_start_token(name, RENAME)
     getter = render_kwargs['getter']
     defaults = getter(render_kwargs['defaults'], name)
@@ -541,6 +621,12 @@ def rename(context, render_kwargs, name=None):
 
 @supports_caller
 def mapping(context, render_kwargs, name=None):
+    """
+    Wrap a block of fields that should be serialized as a dictionary using
+        peppercorn.
+
+    The caller's body should take render_kwargs as an argument.
+    """
     value = make_start_token(name, MAPPING)
     getter = render_kwargs['getter']
     defaults = getter(render_kwargs['defaults'], name)
@@ -580,22 +666,23 @@ def sequence_repeater(context, render_kwargs, name=None, with_adder=False, with_
     template_render_kwargs['getter'] = sequence_getter
     template_render_kwargs['defaults'] = None
     template_render_kwargs['errors'] = None
-    repetition_css_class = _determine_css_class(render_kwargs, 'repetition')
+    repetition_css_class = _to_css_class(_determine_css_classes(render_kwargs,
+            'repetition'))
     template_str = [str(tag('div', class_=repetition_css_class, _closed=False)),
             capture(context, context['caller'].body, template_render_kwargs)]
     if with_removers:
-        remover_css_class = _determine_css_class(render_kwargs, 'remover')
+        remover_css_class = _to_css_class(_determine_css_classes(render_kwargs, 'remover'))
         template_str.append(str(tag('input', type_='button', class_=remover_css_class, _closed=closed, value='Remove')))
     template_str.append('</div>')
     template_str = ''.join(template_str)
-    repeater_css_class = _determine_css_class(render_kwargs, 'repeater')
+    repeater_css_class = _to_css_class(_determine_css_classes(render_kwargs, 'repeater'))
     context.write(tag('div', class_=repeater_css_class, _closed=False, **{'data-template': template_str}))
     if hasattr(context['caller'], 'header'):
         context['caller'].header()
     context.write(tag('input', type='hidden', name='__start__', value=value,
             _closed=closed))
-    repetitions_css_class = _determine_css_class(render_kwargs, 'repetitions')
-    adder_css_class = _determine_css_class(render_kwargs, 'adder')
+    repetitions_css_class = _to_css_class(_determine_css_classes(render_kwargs,
+            'repetitions'))
     context.write(tag('div', class_=repetitions_css_class, _closed=False))
     if defaults:
         for index, new_defaults in enumerate(defaults):
@@ -620,6 +707,7 @@ def sequence_repeater(context, render_kwargs, name=None, with_adder=False, with_
     context.write('</div>')
     context.write(tag('input', type='hidden', name='__end__', value=value, _closed=closed))
     if with_adder:
+        adder_css_class = to_css_class(_determine_css_classes(render_kwargs, 'adder'))
         context.write(tag('input', type_='button', class_=adder_css_class, _closed=closed, value='Add'))
     if hasattr(context['caller'], 'footer'):
         context['caller'].footer()
@@ -629,6 +717,12 @@ def sequence_repeater(context, render_kwargs, name=None, with_adder=False, with_
 
 @supports_caller
 def sequence(context, render_kwargs, name=None):
+    """
+    Wrap a block of fields that should be serialized as a list using
+        peppercorn.
+
+    The caller's body should receive a single argument: render_kwargs.
+    """
     value = make_start_token(name, SEQUENCE)
     getter = render_kwargs['getter']
     defaults = getter(render_kwargs['defaults'], name)
